@@ -1,7 +1,7 @@
 /*
  * Hydrogen
  * Copyright(c) 2002-2008 by Alex >Comix< Cominu [comix@users.sourceforge.net]
- * Copyright(c) 2008-2023 The hydrogen development team [hydrogen-devel@lists.sourceforge.net]
+ * Copyright(c) 2008-2024 The hydrogen development team [hydrogen-devel@lists.sourceforge.net]
  *
  * http://www.hydrogen-music.org
  *
@@ -31,6 +31,7 @@
 #include "Theme.h"
 #include <core/MidiAction.h>
 #include <core/Globals.h>
+#include <core/Helpers/Filesystem.h>
 #include <core/Object.h>
 
 #include <QStringList>
@@ -88,12 +89,15 @@ public:
 	       */
 	      USE_JACK_TRANSPORT = 0,
 	      /**
-	       * Specifies that Hydrogen is using in the time master
-	       * mode and will thus control specific aspects of the
-	       * transport like the overall tempo. Its counterpart is
-	       * #NO_JACK_TIME_MASTER.
+	       * Specifies that Hydrogen should attempt to acquire JACK Timebase
+	       * control.
+	       *
+	       * This represent the state desired by the user. The actual one is
+	       * stored in H2Core::JackAudioDriver::m_timebaseState.
+	       *
+	       * Its counterpart is #NO_JACK_TIMEBASE_CONTROL.
 	       */
-	      USE_JACK_TIME_MASTER = 0,
+	      USE_JACK_TIMEBASE_CONTROL = 0,
 	      SET_PLAY_ON = 0,
 	      BC_ON = 0,/** 
 	       * Specifies whether or not to use JACK transport
@@ -104,11 +108,17 @@ public:
 	       */
 	      NO_JACK_TRANSPORT = 1,
 	      /**
-	       * Specifies that Hydrogen is note using in the time
-	       * master mode. Its counterpart is
-	       * #USE_JACK_TIME_MASTER.
+	       * Specifies that Hydrogen should not be in control of JACK Timebase
+	       * information. This could mean both that there is an external
+	       * application controlling position and tempo of Hydrogen and that
+	       * there are just equal JACK clients.
+	       *
+	       * This represent the state desired by the user. The actual one is
+	       * stored in H2Core::JackAudioDriver::m_timebaseState.
+	       *
+	       * Its counterpart is #USE_JACK_TIMEBASE_CONTROL.
 	       */
-	      NO_JACK_TIME_MASTER = 1,
+	      NO_JACK_TIMEBASE_CONTROL = 1,
 	      SET_PLAY_OFF = 1,
 	      BC_OFF = 1
 	};
@@ -161,6 +171,36 @@ public:
 	std::list<QString> 		m_patternCategories;
 
 	//	audio engine properties ___
+	enum class AudioDriver {
+		None,
+		Null,
+		Fake,
+		Disk,
+		Auto,
+		Jack,
+		Oss,
+		Alsa,
+		PulseAudio,
+		CoreAudio,
+		PortAudio
+	};
+	static AudioDriver parseAudioDriver( const QString& sDriver );
+	static QString audioDriverToQString( const AudioDriver& driver );
+	static std::vector<AudioDriver> getSupportedAudioDrivers();
+
+	/**
+	 * Attempts to call several JACK executables in order to check for
+	 * existing JACK support.
+	 *
+	 * In an earlier version I tried checking the presence of the
+	 * `libjack.so` shared library. But this one comes preinstalled
+	 * with most Linux distribution regardless of JACK itself is
+	 * present or not.
+	 *
+	 * @return Whether or not JACK support appears to be functional.
+	 */
+	static bool checkJackSupport();
+
 	/**
 	 * Audio driver
 	 *
@@ -178,7 +218,7 @@ public:
 	 * - "PulseAudio" : createDriver() will create a PulseAudioDriver.
 	 * - "Fake" : createDriver() will create a FakeDriver.
 	 */
-	QString				m_sAudioDriver;
+	AudioDriver			m_audioDriver;
 	/** If set to true, samples of the metronome will be added to
 	 * #H2Core::AudioEngine::m_songNoteQueue and thus played back on a
 	 * regular basis.*/
@@ -327,54 +367,22 @@ public:
 	/** Specifies which audio settings will be applied to the sample
 		supplied in the JACK per track output ports.*/
 	JackTrackOutputMode		m_JackTrackOutputMode;
-	//jack time master
 
 	/**
-	 * External applications with a faulty JACK timebase master
-	 * implementation can mess up the transport within Hydrogen. To
-	 * guarantee the basic functionality, the user can disable
-	 * timebase support and make Hydrogen only listen to the frame
-	 * number broadcast by the JACK server.
+	 * External applications with a faulty JACK Timebase implementation can mess
+	 * up the transport within Hydrogen. To guarantee the basic functionality,
+	 * the user can disable Timebase support and make Hydrogen only listen to
+	 * the frame number broadcast by the JACK server.
 	 */
 	bool				m_bJackTimebaseEnabled;
 	/**
-	 * Specifies the variable, which has to remain constant in order
-	 * to guarantee a working synchronization and relocation with
-	 * Hydrogen as JACK timebase client.
+	 * Specifies if Hydrogen support the of JACK Timebase protocol. It has two
+	 * states: Preferences::USE_JACK_TIMEBASE_CONTROL and Preferences::NO_JACK_TIMEBASE_CONTROL.
+	 * It is set to Preferences::NO_JACK_TIMEBASE_CONTROL by the
+	 * JackAudioDriver::initTimebaseControl() if Hydrogen couldn't acquire
+	 * Timebase control.
 	 */
-	enum class JackBBTSyncMethod {
-		/** The measure - could be any - does not change during the
-			song.*/
-		constMeasure = 0,
-		/** The length of each pattern must match the measure of the
-			corresponding bar in the timebase master. This way both
-			the pattern position of Hydrogen and the bar information
-			provided by JACK can be assumed to be identical.*/
-		identicalBars = 1 };
-	/**
-	 * Since Hydrogen uses both fixed pattern lengths and recalculates
-	 * the tick size each time it encounters an alternative tempo, its
-	 * transport is incompatible to the BBT information provided by 
-	 * the JACK server. Only if the length of each pattern corresponds
-	 * to the measure of the respective bar in the timebase master
-	 * application, the bar information provided by JACK can be used
-	 * directly to determine chosen pattern. If this, however, is not
-	 * the case - which can quite easily happen - a complete history 
-	 * of all measure and tempo changes would be required to correctly
-	 * identify the pattern. Since this is not provided by JACK, one
-	 * has to either assume the measure or tempo to be constant or 
-	 * that the user took care of adjusting the lengths properly.
-	 */
-	JackBBTSyncMethod	m_JackBBTSync;
-	/**
-	 * Specifies if Hydrogen should run as JACK time master. It
-	 * has two states: Preferences::USE_JACK_TIME_MASTER and
-	 * Preferences::NO_JACK_TIME_MASTER. It is set to
-	 * Preferences::NO_JACK_TIME_MASTER by the
-	 * JackAudioDriver::initTimebaseMaster() if Hydrogen couldn't be
-	 * registered as time master.
-	 */
-	int					m_bJackMasterMode;
+	int					m_bJackTimebaseMode;
 	// ~ jack driver properties
 
 	///Default text editor (used by Playlisteditor)
@@ -400,7 +408,7 @@ public:
 	~Preferences();
 
 	/// Load the preferences file
-	void			loadPreferences( bool bGlobal );
+	bool			loadPreferences( bool bGlobal );
 
 	/// Save the preferences file
 	bool			savePreferences();
@@ -544,6 +552,9 @@ public:
 	WindowProperties	getPlaylistDialogProperties();
 	void				setPlaylistDialogProperties( const WindowProperties& prop );
 
+	WindowProperties	getDirectorProperties();
+	void				setDirectorProperties( const WindowProperties& prop );
+
 	const std::shared_ptr<ColorTheme>	getColorTheme() const;
 	void			setColorTheme( const std::shared_ptr<ColorTheme> pNewColorTheme );
 
@@ -649,8 +660,10 @@ public:
 	int				getExportModeIdx() const;
 	void			setExportModeIdx(int nExportMode);
 	
-	int				getExportTemplateIdx() const;
-	void			setExportTemplateIdx( int nExportTemplateIdx );
+	Filesystem::AudioFormat getExportFormat() const;
+	void			setExportFormat( Filesystem::AudioFormat format );
+	float 			getExportCompressionLevel() const;
+	void			setExportCompressionLevel( float fCompressionLevel );
 
     int				getMidiExportMode() const;
     void			setMidiExportMode(int nExportMode);
@@ -669,6 +682,8 @@ public:
 
 	const std::shared_ptr<Theme> getTheme() const;
 	void setTheme( const std::shared_ptr<Theme> pTheme );
+
+	bool getLoadingSuccessful() const;
 	
 private:
 	/**
@@ -764,6 +779,7 @@ private:
 	WindowProperties		audioEngineInfoProperties;
 	WindowProperties		m_ladspaProperties[MAX_FX];
 	WindowProperties		m_playlistDialogProperties;
+	WindowProperties		m_directorProperties;
 
 	QString					m_sPreferredLanguage;
 
@@ -789,7 +805,8 @@ private:
 	int						m_nExportModeIdx;
 	int						m_nExportSampleRateIdx;
 	int						m_nExportSampleDepthIdx;
-	int						m_nExportTemplateIdx;
+	Filesystem::AudioFormat m_exportFormat;
+	float					m_fExportCompressionLevel;
 	// ~ Export dialog
 
     // Export midi dialog
@@ -816,6 +833,8 @@ private:
 
 	WindowProperties readWindowProperties( XMLNode parent, const QString& windowName, WindowProperties defaultProp );
 	void writeWindowProperties( XMLNode parent, const QString& windowName, const WindowProperties& prop );
+
+	bool m_bLoadingSuccessful;
 };
 
 inline QString			Preferences::getLastExportPatternAsDirectory() const {
@@ -971,14 +990,24 @@ inline void Preferences::setExportSampleRateIdx(int ExportSampleRate)
 	m_nExportSampleRateIdx = ExportSampleRate;
 }
 
-inline int Preferences::getExportTemplateIdx() const
+inline Filesystem::AudioFormat Preferences::getExportFormat() const
 {
-	return m_nExportTemplateIdx;
+	return m_exportFormat;
 }
 
-inline void Preferences::setExportTemplateIdx(int ExportTemplateIdx)
+inline void Preferences::setExportFormat( Filesystem::AudioFormat format )
 {
-	m_nExportTemplateIdx = ExportTemplateIdx;
+	m_exportFormat = format;
+}
+
+inline float Preferences::getExportCompressionLevel() const
+{
+	return m_fExportCompressionLevel;
+}
+
+inline void Preferences::setExportCompressionLevel( float fCompressionLevel )
+{
+	m_fExportCompressionLevel = fCompressionLevel;
 }
 
 inline const QString& Preferences::getDefaultEditor() {
@@ -1333,6 +1362,13 @@ inline void Preferences::setPlaylistDialogProperties( const WindowProperties& pr
 	m_playlistDialogProperties = prop;
 }
 
+inline WindowProperties Preferences::getDirectorProperties() {
+	return m_directorProperties;
+}
+inline void Preferences::setDirectorProperties( const WindowProperties& prop ) {
+	m_directorProperties = prop;
+}
+
 inline const std::shared_ptr<ColorTheme> Preferences::getColorTheme() const {
 	return m_pTheme->getColorTheme();
 }
@@ -1452,6 +1488,9 @@ inline void Preferences::setTheme( const std::shared_ptr<Theme> pTheme ) {
 inline const std::shared_ptr<Theme> Preferences::getTheme() const {
 	return m_pTheme;
 }
+inline bool Preferences::getLoadingSuccessful() const {
+	return m_bLoadingSuccessful;
+}
 };
 
 #endif
